# coding=utf-8

#  Copyright (c) 2017 Presslabs SRL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import pytest
from mock import patch, MagicMock

from freezegun import freeze_time
from payu.payments import TokenPayment, ALUPayment
from payu.conf import PAYU_TOKENS_URL, PAYU_ALU_URL


@pytest.mark.parametrize('order, key, signature', [
    ({
        "AMOUNT": 1,
        "CURRENCY": "RON",
        "BILL_ADDRESS": "address 1",
        "BILL_CITY": "Iasi",
        "BILL_EMAIL": "john@doe.com",
        "BILL_FNAME": "John",
        "BILL_LNAME": "Doe",
        "BILL_PHONE": "0243236298",
        "DELIVERY_ADDRESS": "address 2",
        "DELIVERY_CITY": "Suceava",
        "DELIVERY_EMAIL": "john@doe.com",
        "DELIVERY_FNAME": "John",
        "DELIVERY_LNAME": "Doe",
        "DELIVERY_PHONE": "0243236298",
        "EXTERNAL_REF": "25787sa1"
    }, "123", "c6d24967498508cbfeefa26095613716"),
    ({
        "AMOUNT": 1,
        "CURRENCY": "RON",
        "BILL_EMAIL": "john@doe.com",
        "BILL_LNAME": "Doe",
        "DELIVERY_CITY": "Suceava",
        "DELIVERY_EMAIL": "john@doe.com",
        "EXTERNAL_REF": "25787sa1"
    }, "-1", "e3103c723372424b2b8292bbf6fcb436"),
    ({
         "AMOUNT": 1,
         "CURRENCY": "MXN",
         "BILL_EMAIL": "juan@jose.com",
         "BILL_LNAME": "Jose",
         "DELIVERY_CITY": "Ciudad de México".decode('utf-8'),
         "EXTERNAL_REF": "25787sa1"
     }, "-1", "0c79288878d66cd85c02c09664547625"),
    ({
        "AMOUNT": 1,
    }, "", "bef91610dda7aabfe371623edb399f3e")
])
def test_payment_signature(order, key, signature):
    assert TokenPayment.get_signature(order, key) == signature
    assert ALUPayment.get_signature(order, key) == signature


@patch('payu.payments.requests')
def test_token_pay(mocked_requests):
    payment = TokenPayment("", "")
    payment._build_payload = MagicMock(return_value="expected_payload")

    expected_response = "ok"
    mocked_requests.post.return_value = MagicMock(content=expected_response)

    assert payment.pay() == expected_response
    mocked_requests.post.assert_called_once_with(PAYU_TOKENS_URL,
                                                 data="expected_payload")


@patch('payu.payments.datetime')
def test_token_build_payload(mocked_datetime):
    payment = TokenPayment({"AMOUNT": 1}, "token", "key", "test")

    mocked_datetime.now.return_value = MagicMock(strftime=MagicMock(return_value="now"))
    assert payment._build_payload() == {
        'AMOUNT': 1,
        'MERCHANT': 'test',
        'METHOD': 'TOKEN_NEWSALE',
        'REF_NO': 'token',
        'SIGN': '4192969bae28a16ba4777354903f895a',
        'TIMESTAMP': 'now'
    }


@patch('payu.payments.requests')
def test_alu_token_pay(mocked_requests):
    payment = ALUPayment("", "")
    payment._build_payload = MagicMock(return_value="expected_payload")

    expected_response = "ok"
    mocked_requests.post.return_value = MagicMock(content=expected_response)

    assert payment.pay() == expected_response
    mocked_requests.post.assert_called_once_with(PAYU_ALU_URL,
                                                 data="expected_payload")


@freeze_time("2020-06-30 11:49:52")
def test_alutoken_build_payload():
    payment = ALUPayment(
        {
            "AMOUNT": 1,
            "ORDER": [
                {
                    'PNAME': 'CD Player',
                    'PCODE': 'PROD_04891',
                    'PINFO': 'Extended Warranty - 5 Years',
                    'PRICE': '82.3',
                    'PRICE_TYPE': 'GROSS',
                    'QTY': '7',
                    'VAT':'20'
                }
            ],
        }, "token", "key", "test")

    assert payment._build_payload() == {
        'AMOUNT': 1,
        'MERCHANT': 'test',
        'CC_TOKEN': 'token',
        'ORDER_DATE': '2020-06-30 11:49:52',
        'ORDER_HASH': 'a25afe5b6fd8246e7b8c8846535ba01b',
        'ORDER_PCODE[0]': 'PROD_04891',
        'ORDER_PINFO[0]': 'Extended Warranty - 5 Years',
        'ORDER_PNAME[0]': 'CD Player',
        'ORDER_PRICE[0]': '82.3',
        'ORDER_PRICE_TYPE[0]': 'GROSS',
        'ORDER_QTY[0]': '7',
        'ORDER_VAT[0]': '20'
    }



#
# Copyright 2012-2016 PressLabs SRL
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#

import hmac
import hashlib
from datetime import datetime

import pytz

from django.http import HttpResponse, QueryDict
from django.utils.six import text_type
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_http_methods

from payu.conf import PAYU_MERCHANT_KEY, PAYU_IPN_FIELDS
from payu.models import PayUIPN, PayUToken, PayUIDN
from payu.forms import PayUIPNForm


@csrf_exempt
@require_http_methods(["GET", "POST"])
def ipn(request):
    ipn_obj = None
    error = None

    form_data = request.POST
    ipn_form = PayUIPNForm(form_data)

    validation_hash = text_type()
    for field in PAYU_IPN_FIELDS:
        if field not in request.POST:
            continue

        field_value = request.POST.getlist(field)

        validation_hash += text_type().join(
            [text_type('{length}{value}').format(
                length=len(text_type(value).encode('utf-8')), value=value
            ) for value in field_value]
        ).encode('utf-8')

    expected_hash = hmac.new(PAYU_MERCHANT_KEY, validation_hash, hashlib.md5).hexdigest()
    request_hash = request.POST.get('HASH', '')

    if request_hash != expected_hash:
        error = 'Invalid hash %s. Hash string \n%s' % (request_hash, expected_hash)
    else:
        if ipn_form.is_valid():
            try:
                ipn_obj = ipn_form.save(commit=False)
            except Exception, exception:
                error = "Exception while processing. (%s)" % exception
        else:
            error = "Invalid form. (%s)" % ipn_form.errors

    if not ipn_obj:
        ipn_obj = PayUIPN()

    # Set query params and sender's IP address
    ipn_obj.response = getattr(request, request.method).urlencode()
    ipn_obj.ip_address = request.META.get('REMOTE_ADDR', '')

    if error:
        # We save errors in the error field
        ipn_obj.set_flag(error)

    ipn_obj.save()

    # Check for a token in the request and save it if found
    IPN_CC_TOKEN = request.POST.get('IPN_CC_TOKEN')
    TOKEN_HASH = request.POST.get('TOKEN_HASH')
    IPN_CC_MASK = request.POST.get('IPN_CC_MASK')
    IPN_CC_EXP_DATE = request.POST.get('IPN_CC_EXP_DATE')

    if all([IPN_CC_TOKEN, IPN_CC_MASK, IPN_CC_EXP_DATE]):
        PayUToken.objects.create(
            IPN_CC_TOKEN=IPN_CC_TOKEN,
            IPN_CC_MASK=IPN_CC_MASK,
            IPN_CC_EXP_DATE=IPN_CC_EXP_DATE,
            TOKEN_HASH=TOKEN_HASH,
            ipn=ipn_obj
        )

    PayUIDN.objects.create(ipn=ipn_obj)

    # Send confirmation to PayU that we received this request
    date = datetime.now(pytz.UTC).strftime('%Y%m%d%H%M%S')

    confirmation_hash = ""
    for field in ["IPN_PID[]", "IPN_PNAME[]", "IPN_DATE"]:
        field_value = request.POST.getlist(field)

        if not field_value:
            confirmation_hash += "0"
        else:
            confirmation_hash += field_value

    confirmation_hash = hmac.new(PAYU_MERCHANT_KEY,
                                 '%s14%s' % (confirmation_hash, date)).hexdigest()
    return HttpResponse('<EPAYMENT>%s|%s</EPAYMENT>' % (date, confirmation_hash))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='PayUIDN',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('sent', models.BooleanField(default=False)),
                ('success', models.BooleanField(default=False)),
                ('response', models.TextField(blank=True)),
            ],
            options={
                'verbose_name': 'PayU IDN',
            },
        ),
        migrations.CreateModel(
            name='PayUIPN',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('REFNO', models.CharField(max_length=9, verbose_name=b'ePayment reference')),
                ('REFNOEXT', models.CharField(max_length=100, verbose_name=b'Merchant reference')),
                ('ORDERNO', models.CharField(max_length=6, verbose_name=b'Merchant order #')),
                ('ORDERSTATUS', models.CharField(max_length=18, verbose_name=b'Status', choices=[(b'PAYMENT_AUTHORIZED', b'PAYMENT_AUTHORIZED'), (b'PAYMENT_RECEIVED', b'PAYMENT_RECEIVED'), (b'TEST', b'TEST'), (b'CASH', b'CASH'), (b'COMPLETE', b'COMPLETE'), (b'REVERSED', b'REVERSED'), (b'REFUND', b'REFUND')])),
                ('HASH', models.CharField(max_length=64)),
                ('PAYMETHOD_CODE', models.CharField(max_length=10, verbose_name=b'Payment method code')),
                ('SALEDATE', models.DateTimeField(null=True, verbose_name=b'Sale date', blank=True)),
                ('PAYMENTDATE', models.DateTimeField(null=True, verbose_name=b'Payment date', blank=True)),
                ('PAYMETHOD', models.CharField(max_length=100, null=True, verbose_name=b'Payment method', blank=True)),
                ('FIRSTNAME', models.CharField(help_text=b"Client's first name", max_length=40, null=True, blank=True)),
                ('LASTNAME', models.CharField(help_text=b"Client's last name", max_length=40, null=True, blank=True)),
                ('IDENTITY_NO', models.CharField(help_text=b'Customer ID Card series and                                    number (Series / Number - available                                    only for Romanian customers)', max_length=15, null=True, blank=True)),
                ('IDENTITY_ISSUER', models.CharField(help_text=b'IDENTITY_NO ID Card                                        issuer authority ', max_length=100, null=True, blank=True)),
                ('CARD_TYPE', models.CharField(help_text=b'Used credit card type.                                  Ex: "Visa" or "MasterCard"', max_length=100, null=True, blank=True)),
                ('IDENTITY_CNP', models.CharField(help_text=b"Customer's personal numeric                                     code, available only for Romanian customers.", max_length=13, null=True, blank=True)),
                ('COMPANY', models.CharField(help_text=b'Company name (maximum length:                                              40 characters) ', max_length=40, null=True, blank=True)),
                ('REGISTRATIONNUMBER', models.CharField(help_text=b"Company's Commerce                                           Registry registration number                                           (maximum length: 40 characters)", max_length=40, null=True, blank=True)),
                ('FISCALCODE', models.CharField(help_text=b'Unique Registration Number /                                   VAT ID (maximum length: 40 characters)', max_length=40, null=True, blank=True)),
                ('CBANKNAME', models.CharField(help_text=b"Company's Bank (maximum                                  length: 40 characters) ", max_length=40, null=True, blank=True)),
                ('CBANKACCOUNT', models.CharField(help_text=b"Company's Bank Account                                     (maximum length: 50 characters)", max_length=50, null=True, blank=True)),
                ('ADDRESS1', models.CharField(help_text=b'Address (maximum length: 100                                 characters)', max_length=100, null=True, blank=True)),
                ('ADDRESS2', models.CharField(help_text=b'Additional Address info                                 (maximum length: 100 characters)', max_length=100, null=True, blank=True)),
                ('CITY', models.CharField(help_text=b'City (maximum length: 30 characters)', max_length=30, null=True, blank=True)),
                ('STATE', models.CharField(help_text=b'State/Sector/County (maximum                              length: 30 characters)', max_length=30, null=True, blank=True)),
                ('ZIPCODE', models.CharField(help_text=b'ZIP/Postal Code (maximum length:                                20 characters)', max_length=20, null=True, blank=True)),
                ('COUNTRY', models.CharField(help_text=b'Country (maximum length: 50                                characters)', max_length=50, null=True, blank=True)),
                ('COUNTRY_CODE', models.CharField(help_text=b'Country (maximum length: 10                                     characters)', max_length=10, null=True, blank=True)),
                ('PHONE', models.CharField(help_text=b'Phone number (maximum length: 40                              characters)', max_length=40, null=True, blank=True)),
                ('FAX', models.CharField(help_text=b'Fax number (maximum length: 40                            characters)', max_length=40, null=True, blank=True)),
                ('CUSTOMEREMAIL', models.CharField(help_text=b"Customer's e-mail address                                      (maximum length: 40 characters)", max_length=40, null=True, blank=True)),
                ('FIRSTNAME_D', models.CharField(help_text=b'First name (maximum length:                                    40 characters) ', max_length=40, null=True, blank=True)),
                ('LASTAME_D', models.CharField(help_text=b'Last Name (maximum length: 40                                  characters)', max_length=40, null=True, blank=True)),
                ('COMPANY_D', models.CharField(help_text=b'Company (maximum length: 50                                  characters)', max_length=50, null=True, blank=True)),
                ('ADDRESS1_D', models.CharField(help_text=b'Address (maximum length: 100                                   characters)', max_length=100, null=True, blank=True)),
                ('ADDRESS2_D', models.CharField(help_text=b'Additional address info                                   (maximum length: 100 characters)', max_length=100, null=True, blank=True)),
                ('CITY_D', models.CharField(help_text=b'City (maximum length: 30                               characters)', max_length=30, null=True, blank=True)),
                ('STATE_D', models.CharField(help_text=b'State/Sector/County (maximum                                length: 30 characters)', max_length=30, null=True, blank=True)),
                ('ZIPCODE_D', models.CharField(help_text=b'ZIP/Postal Code (maximum                                  length: 20 characters)', max_length=20, null=True, blank=True)),
                ('COUNTRY_D', models.CharField(help_text=b'Country (maximum length: 50                                  characters)', max_length=50, null=True, blank=True)),
                ('COUNTRY_D_CODE', models.CharField(help_text=b'Country (maximum length:                                       10 characters)', max_length=10, null=True, blank=True)),
                ('PHONE_D', models.CharField(help_text=b'Phone number (maximum length: 40                                characters)', max_length=40, null=True, blank=True)),
                ('EMAIL_D', models.CharField(help_text=b'E-mail (maximum length: 40                                characters)', max_length=40, null=True, blank=True)),
                ('IPADDRESS', models.CharField(help_text=b"Client's IP Address (maximum                                  length: 250 characters)", max_length=250, null=True, blank=True)),
                ('IPCOUNTRY', models.CharField(help_text=b"Client's IP Country (maximum                                  length: 50 characters)", max_length=50, null=True, blank=True)),
                ('COMPLETE_DATE', models.CharField(help_text=b'The order completion date,                                      in the following format: Y-m-d H:i:s                                      (2012-04-26 15:02:28) .', max_length=40, null=True, blank=True)),
                ('CURRENCY', models.CharField(help_text=b'The currency in which the order                                 has been processed. Possible values: RON,                                 USD, EUR.', max_length=10, null=True, blank=True)),
                ('LANGUAGE', models.CharField(help_text=b'The language in which the order                                 has been processed. Possible values: ro,                                 en, fr, de, it.', max_length=40, null=True, blank=True)),
                ('IPN_PID', models.TextField(help_text=b'Array with the ID Codes of the                                ordered products, in the PayU database (PayU                                reference) ', null=True, blank=True)),
                ('IPN_PNAME', models.TextField(help_text=b'Array with product names ', null=True, blank=True)),
                ('IPN_PCODE', models.TextField(help_text=b'Array with the product codes                                  assigned by the vendor in the system (vendor                                  reference)', null=True, blank=True)),
                ('IPN_INFO', models.TextField(help_text=b'Array with additional                                 information sent for each ordered product (if                                 they have been sent to PayU)', null=True, blank=True)),
                ('IPN_QTY', models.TextField(help_text=b'Array with the product quantities', null=True, blank=True)),
                ('IPN_PRICE', models.TextField(help_text=b'Array with unit prices per                                  product (without VAT), in RON, with                                  period/full-stop (.) as decimal place                                  separator', null=True, blank=True)),
                ('IPN_VAT', models.TextField(help_text=b'Array with VAT values per                                product, with period "." as decimal place                                separator', null=True, blank=True)),
                ('IPN_VER', models.TextField(help_text=b'Array with product versions                                (maximum length: 50 characters)', null=True, blank=True)),
                ('IPN_DISCOUNT', models.TextField(help_text=b'Array with the amounts with                                     which there has been made a discount                                     in a promotion. Including VAT. ', null=True, blank=True)),
                ('IPN_PROMONAME', models.TextField(help_text=b'Array with the names of the                                      promotions in which the discounts                                      specified above have been made.', null=True, blank=True)),
                ('IPN_PROMOCODE', models.TextField(help_text=b'Array with the code of the                                      promotions in which the discounts                                      specified above have been made.', null=True, blank=True)),
                ('IPN_ORDER_COSTS', models.TextField(help_text=b"Array with costs for each                                        product from order (expressed in                                        order's currency)", null=True, blank=True)),
                ('IPN_REC_CURRENT_ITERATION_NO', models.TextField(help_text=b'Current recurring                                                                   period (avaible                                                                   only for recurrent                                                                   payments)', null=True, blank=True)),
                ('IPN_REC_ORIGINAL_REFNO', models.TextField(help_text=b'Array containing                                                             the reference to                                                             the original order', null=True, blank=True)),
                ('IPN_REC_INTERVAL', models.TextField(help_text=b'Array containing                                                       recurring intervals                                                       (day/month/week) for                                                       each order', null=True, blank=True)),
                ('IPN_REC_EXPIRATION_DATE', models.TextField(help_text=b'Array with                                                              expiration dates                                                              for each                                                              recurrence', null=True, blank=True)),
                ('IPN_REC_MULTIPLIER', models.TextField(help_text=b'Array with reccurence                                                         period (interval x                                                         multiplier) for each                                                         product from the order', null=True, blank=True)),
                ('IPN_DELIVEREDCODES', models.TextField(help_text=b'Array with the codes                                                         delivered to the                                                         clients, if the PayU                                                         contract contains this                                                         feature. Each element                                                         in the array is                                                         represented by a                                                         string, having comma                                                         (,) as a separator for                                                         each sent code, in                                                         case the ordered                                                         quantity is greater                                                         than 1.', null=True, blank=True)),
                ('IPN_DOWNLOAD_LINK', models.CharField(help_text=b'Download link of the                                                        product delivered to                                                        the client', max_length=250, null=True, blank=True)),
                ('IPN_TOTAL', models.TextField(help_text=b'Partial total on order line                                                (including VAT), with                                                period/full-stop (.) as a                                                decimal place separator', null=True, blank=True)),
                ('IPN_TOTALGENERAL', models.CharField(help_text=b'Total transaction                                                       amount, including VAT                                                       costs, with                                                       period/full-stop (.) as                                                       a decimal place                                                       separator', max_length=40, null=True, blank=True)),
                ('IPN_SHIPPING', models.CharField(help_text=b'Total amount paid for                                                   shippment', max_length=50, null=True, blank=True)),
                ('IPN_REFERRER', models.CharField(help_text=b'HTTP referrer of the sale.', max_length=250, null=True, blank=True)),
                ('IPN_GLOBALDISCOUNT', models.CharField(help_text=b'Global discount of the                                                         sale. This field is                                                         option and is avaible                                                         only if the amount is                                                         greater than zero.', max_length=250, null=True, blank=True)),
                ('IPN_COMMISSION', models.CharField(help_text=b"Payu's commision in RON,                                                     with period/full-stop (.)                                                     as a decimal place                                                     separator.", max_length=50, null=True, blank=True)),
                ('IPN_DATE', models.DateTimeField(help_text=b"IPN POST's sending date in the                                                following format: YmdHMS (ex.:                                                20120426145935)", max_length=40, null=True, blank=True)),
                ('response', models.TextField(blank=True)),
                ('ip_address', models.GenericIPAddressField(null=True, blank=True)),
                ('flag', models.BooleanField(default=False)),
                ('flag_info', models.TextField(blank=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'db_table': 'payu_ipn',
                'verbose_name': 'PayU IPN',
            },
        ),
        migrations.CreateModel(
            name='PayUToken',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('IPN_CC_TOKEN', models.CharField(max_length=9, verbose_name=b'Token')),
                ('IPN_CC_MASK', models.CharField(max_length=36, verbose_name=b'Last 4 digits')),
                ('IPN_CC_EXP_DATE', models.DateField(verbose_name=b'Expiration date')),
                ('ipn', models.OneToOneField(to='payu.PayUIPN')),
            ],
            options={
                'verbose_name': 'PayU Tokens V1',
            },
        ),
        migrations.AddField(
            model_name='payuidn',
            name='ipn',
            field=models.OneToOneField(to='payu.PayUIPN'),
        ),
    ]



# Copyright (c) 2017 Presslabs SRL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from django.db.models import Q
from django.core.management.base import BaseCommand

from payu.models import PayUIDN

logger = logging.getLogger(__name__)


def string_to_list(list_as_string):
    return list(map(int, list_as_string.strip('[] ').split(',')))


class Command(BaseCommand):
    help = ''

    def add_arguments(self, parser):
        parser.add_argument(
            '--idns',
            help='A list of idns pks to be sent.',
            action='store', dest='idns', type=string_to_list
        )

    def handle(self, *args, **options):
        idns = PayUIDN.objects.filter(Q(sent=False) | Q(sent=True, success=False))

        if options['idns']:
            idns = idns.filter(pk__in=options['idns'])

        for idn in idns:
            idn.send()

            if not idn.success:
                logger.error('Encountered exception while processing idn'
                             'with id=%s, ex=%s.', idn.id, idn.response,
                             exc_info=True)



# Copyright (c) 2017 Presslabs SRL
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import hmac
from datetime import datetime

import requests

from django.utils.six import text_type

from payu.conf import (PAYU_MERCHANT_KEY, PAYU_MERCHANT,
                       PAYU_ALU_URL, PAYU_TOKENS_URL)


class BasePayment(object):
    def __init__(self, order, token, merchant_key=PAYU_MERCHANT_KEY,
                 merchant=PAYU_MERCHANT):

        self.order = order
        self.token = token
        self.merchant_key = merchant_key
        self.merchant = merchant

    def pay(self):
        raise NotImplementedError

    @classmethod
    def get_signature(self, payload, merchant_key):
        sorted_payload = sorted(payload.items(), key=lambda item: item[0])
        parameters = text_type().join(
            [text_type('{length}{value}').format(
                length=len(text_type(parameter[1]).encode('utf-8')), value=parameter[1]
            ) for parameter in sorted_payload]
        ).encode('utf-8')
        return hmac.new(merchant_key, parameters).hexdigest()


class TokenPayment(BasePayment):
    def pay(self):
        return requests.post(PAYU_TOKENS_URL,
                             data=self._build_payload()).content

    def _build_payload(self):
        payload = {
            'REF_NO': self.token,
            'METHOD': 'TOKEN_NEWSALE',
            'MERCHANT': self.merchant,
            'TIMESTAMP': datetime.now().strftime("%Y%m%d%H%M%S"),
        }

        payload.update(self.order)

        payload['SIGN'] = self.get_signature(payload, self.merchant_key)

        return payload


class ALUPayment(BasePayment):
    def __init__(self, *args, **kwargs):
        self.lu_token_type = kwargs.pop("lu_token_type", None)

        super(ALUPayment, self).__init__(*args, **kwargs)

    def pay(self):
        return requests.post(PAYU_ALU_URL, data=self._build_payload()).content

    def _build_payload(self):
        order = self.order
        order["MERCHANT"] = self.merchant
        if not order.get("ORDER_DATE"):
            order["ORDER_DATE"] = datetime.now().strftime('%Y-%m-%d %H:%M:%S')

        payload = self._parse_orders(order.pop('ORDER'))
        payload['CC_TOKEN'] = self.token
        if self.lu_token_type and payload['CC_TOKEN']:
            payload['CC_CVV'] = ""
            payload['LU_TOKEN_TYPE'] = self.lu_token_type

        payload.update(**order)
        payload["ORDER_HASH"] = ALUPayment.get_signature(payload,
                                                         self.merchant_key)

        return payload

    def _parse_orders(self, orders):
        """
        Transform orders from list objects to PHP arrays:
            [
                {
                    'PNAME': 'CD Player',
                    'PCODE': 'PROD_04891',
                    'PINFO': 'Extended Warranty - 5 Years',
                    'PRICE': '82.3',
                    'PRICE_TYPE': 'GROSS',
                    'QTY': '7',
                    'VAT':'20'
                },
                {
                    'PNAME': 'Mobile Phone',
                    'PCODE': 'PROD_07409',
                    'PINFO': 'Dual SIM',
                    'PRICE': '1945.75',
                    'PRICE_TYPE': 'GROSS',
                    'QTY': '3',
                    'VAT':'20'
                },
                {
                    'PNAME': 'Laptop',
                    'PCODE': 'PROD_04965',
                    'PINFO': '17" Display',
                    'PRICE': '5230',
                    'PRICE_TYPE': 'GROSS',
                    'QTY': '1',
                    'VAT':'20'
                }
            ]

                ||
                \/

            {
                 'ORDER_PCODE[0]': 'PROD_04891',
                 'ORDER_PCODE[1]': 'PROD_07409',
                 'ORDER_PCODE[2]': 'PROD_04965',
                 'ORDER_PINFO[0]': 'Extended Warranty - 5 Years',
                 'ORDER_PINFO[1]': 'Dual SIM',
                 'ORDER_PINFO[2]': '17" Display',
                 'ORDER_PNAME[0]': 'CD Player',
                 'ORDER_PNAME[1]': 'Mobile Phone',
                 'ORDER_PNAME[2]': 'Laptop',
                 'ORDER_PRICE[0]': '82.3',
                 'ORDER_PRICE[1]': '1945.75',
                 'ORDER_PRICE[2]': '5230',
                 'ORDER_PRICE_TYPE[0]': 'GROSS',
                 'ORDER_PRICE_TYPE[1]': 'GROSS',
                 'ORDER_PRICE_TYPE[2]': 'GROSS',
                 'ORDER_QTY[0]': '7',
                 'ORDER_QTY[1]': '3',
                 'ORDER_QTY[2]': '1',
                 'ORDER_VAT[0]': '20',
                 'ORDER_VAT[1]': '20',
                 'ORDER_VAT[2]': '20'
            }
        """

        result = {}

        for index, order in enumerate(orders):
            for detail, value in order.iteritems():
                result["ORDER_%s[%s]" % (detail, index)] = value

        return result



# coding=utf-8
#
# Copyright 2012-2016 PressLabs SRL
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
from django.contrib import admin

from payu.models import PayUIPN, PayUToken, PayUIDN


class PayUIPNAdmin(admin.ModelAdmin):
    list_display = ('__unicode__', 'REFNOEXT', 'ORDERSTATUS', 'flag',
                    'flag_info', 'created_at')
    list_filter = ('ORDERSTATUS', 'flag')


class PayUTokenAdmin(admin.ModelAdmin):
    list_display = ('__unicode__', 'ipn', 'IPN_CC_TOKEN', 'IPN_CC_MASK',
                    'IPN_CC_EXP_DATE')


class PayUIDNAdmin(admin.ModelAdmin):
    list_display = ('ipn', 'sent', 'success')


admin.site.register(PayUIPN, PayUIPNAdmin)
admin.site.register(PayUToken, PayUTokenAdmin)
admin.site.register(PayUIDN, PayUIDNAdmin)



# coding=utf-8
#
# Copyright 2012-2016 PressLabs SRL
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
from django.conf import settings


PAYU_MERCHANT = getattr(settings, 'PAYU_MERCHANT', '')
PAYU_MERCHANT_KEY = getattr(settings, 'PAYU_KEY', '')
PAYU_TEST_TRANSACTION = getattr(settings, 'PAYU_TEST', 'TRUE')
PAYU_LU_CALLBACK = getattr(settings, 'PAYU_LU_CALLBACK', '')


PAYU_LU_URL = getattr(settings, 'PAYU_LU_URL',
                      'https://secure.payu.ro/order/lu.php')
PAYU_ALU_URL = getattr(settings, 'PAYU_ALU_URL',
                       'https://secure.payu.ro/order/alu/v3')
PAYU_IDN_URL = getattr(settings, 'PAYU_IDN_URL',
                       'https://secure.payu.ro/order/idn.php')

PAYU_TOKENS_URL = getattr(settings, 'PAYU_TOKENS_URL',
                          'https://secure.payu.ro/order/tokens/')

PAYU_ORDER_DETAILS = ['PNAME', 'PGROUP', 'PCODE', 'PINFO', 'PRICE', 'PRICE_TYPE',
                      'QTY', 'VAT', 'VER']

PAYU_ORDER_DETAILS_DEFAULTS = {
    'QTY': 1,
    'VAT': 24
}

PAYU_DATE_FORMATS = (
    '%Y-%m-%d %H:%M:%S'
)

PAYU_CURRENCIES = (
    ('USD', 'USD'),
    ('RON', 'RON'),
    ('EUR', 'EUR')
)

PAYU_PAYMENT_METHODS = (
    ('CCVISAMC', 'VISA/Mastercard Card'),
    ('CCAMEX', 'AMEX Card'),
    ('CCDINERS', 'Diners Club Card'),
    ('CCJCB', 'JCB Card'),
    ('WIRE', 'Bank Wire'),
    ('PAYPAL', 'PayPal')
)

PAYU_LANGUAGES = (
    ('RO', u'Română'),
    ('EN', u'English'),
    ('DE', u'Deutsch'),
    ('ES', u'Español'),
    ('FR', u'Français'),
    ('IT', u'Italiano')
)

PAYU_PAYMENT_STATUS = (
    ('PAYMENT_AUTHORIZED', 'PAYMENT_AUTHORIZED'),
    ('PAYMENT_RECEIVED', 'PAYMENT_RECEIVED'),
    ('TEST', 'TEST'),
    ('CASH', 'CASH'),
    ('COMPLETE', 'COMPLETE'),
    ('REVERSED', 'REVERSED'),
    ('REFUND', 'REFUND')
)

PAYU_IPN_FIELDS = ['SALEDATE', 'PAYMENTDATE', 'REFNO', 'REFNOEXT', 'ORDERNO',
                   'ORDERSTATUS', 'PAYMETHOD', 'PAYMETHOD_CODE', 'FIRSTNAME',
                   'LASTNAME', 'IDENTITY_NO', 'IDENTITY_ISSUER', 'CARD_TYPE',
                   'IDENTITY_CNP', 'COMPANY', 'REGISTRATIONNUMBER',
                   'FISCALCODE', 'CBANKNAME', 'CBANKACCOUNT', 'ADDRESS1',
                   'ADDRESS2', 'CITY', 'STATE', 'ZIPCODE', 'COUNTRY',
                   'COUNTRY_CODE', 'PHONE', 'FAX', 'CUSTOMEREMAIL',
                   'FIRSTNAME_D', 'LASTNAME_D', 'COMPANY_D', 'ADDRESS1_D',
                   'ADDRESS2_D', 'CITY_D', 'STATE_D', 'ZIPCODE_D', 'COUNTRY_D',
                   'COUNTRY_D_CODE', 'PHONE_D', 'EMAIL_D', 'IPADDRESS',
                   'IPCOUNTRY', 'COMPLETE_DATE', 'CURRENCY', 'LANGUAGE',
                   'IPN_PID[]', 'IPN_PNAME[]', 'IPN_PCODE[]', 'IPN_INFO[]',
                   'IPN_QTY[]', 'IPN_PRICE[]', 'IPN_VAT[]', 'IPN_VER[]',
                   'IPN_DISCOUNT[]', 'IPN_PROMONAME[]', 'IPN_PROMOCODE[]',
                   'IPN_ORDER_COSTS[]', 'IPN_REC_CURRENT_ITERATION_NO[]',
                   'IPN_REC_ORIGINAL_REFNO[]', 'IPN_REC_INTERVAL[]',
                   'IPN_REC_EXPIRATION_DATE[]', 'IPN_REC_MULTIPLIER[]',
                   'IPN_DELIVEREDCODES[]', 'IPN_DOWNLOAD_LINK', 'IPN_TOTAL[]',
                   'IPN_TOTALGENERAL', 'IPN_SHIPPING', 'IPN_REFERRER',
                   'IPN_GLOBALDISCOUNT', 'IPN_COMMISSION', 'IPN_DATE',
                   'IPN_CC_TOKEN', 'IPN_CC_MASK', 'IPN_CC_EXP_DATE', 'TOKEN_HASH']
